@@ -21,8 +21,8 @@
 
 #include <glib.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
-#include "mountlist.h"
 
 #if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
 #include <string.h>
@@ -33,7 +33,18 @@
 #include <glibtop.h>
 #include <glibtop/mountlist.h>
 
-static struct mount_entry *read_filesystem_list (gboolean need_fs_type);
+/* A mount table entry. */
+struct mount_entry
+{
+  char *me_devname;             /* Device node pathname, including "/dev/". */
+  char *me_mountdir;            /* Mount point directory pathname. */
+  char *me_type;                /* "nfs", "4.2", etc. */
+  dev_t me_dev;                 /* Device number of me_mountdir. */
+  struct mount_entry *me_next;
+};
+
+
+static struct mount_entry *read_filesystem_list (void);
 
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -207,12 +218,10 @@ fstype_to_string (int t)
 
 /* Return a list of the currently mounted filesystems, or NULL on error.
    Add each entry to the tail of the list so that they stay in order.
-   If NEED_FS_TYPE is nonzero, ensure that the filesystem type fields in
-   the returned list are valid.  Otherwise, they might not be.
 */
 
 static struct mount_entry *
-read_filesystem_list (gboolean need_fs_type)
+read_filesystem_list (void)
 {
   struct mount_entry *mount_list;
   struct mount_entry *me;
@@ -396,9 +405,8 @@ read_filesystem_list (gboolean need_fs_type)
 #endif
 	me->me_mountdir = g_strdup (mnt.mt_filsys);
 	me->me_dev = (dev_t) -1;	/* Magic; means not known yet. */
-	me->me_type = g_strdup ("");
+
 #ifdef GETFSTYP			/* SVR3.  */
-	if (need_fs_type)
 	  {
 	    struct statfs fsd;
 	    char typebuf[FSTYPSZ];
@@ -407,7 +415,10 @@ read_filesystem_list (gboolean need_fs_type)
 		&& sysfs (GETFSTYP, fsd.f_fstyp, typebuf) != -1)
 	      me->me_type = g_strdup (typebuf);
 	  }
+#else
+	me->me_type = g_strdup ("");
 #endif
+
 	me->me_next = NULL;
 
 	/* Add to the linked list. */
@@ -525,27 +536,28 @@ read_filesystem_list (gboolean need_fs_type)
 
 static gboolean ignore_mount_entry(const struct mount_entry *me)
 {
+	/* keep sorted */
 	static const char ignored[][12] = {
+		"autofs",
+		"binfmt_misc",
+		"devpts",
+		"mntfs",
+		"none",
+		"openpromfs",
 		"proc",
 		"procfs",
-		"autofs",
+		"supermount",
 		"sysfs",
-		"none",
-		"devpts",
+		"unknown",
 		"usbdevfs",
-		"binfmt_misc",
-		"supermount"
+		"usbfs"
 	};
 
-	const char (*i)[12] = &ignored[0];
+	typedef int (*Comparator)(const void*, const void*);
 
-	while(i != (&ignored[0] + G_N_ELEMENTS(ignored))) {
-		if(strcmp(*i, me->me_type) == 0)
-			return TRUE;
-		++i;
-	}
-
-	return FALSE;
+	return bsearch(me->me_type,
+		       ignored, G_N_ELEMENTS(ignored), sizeof ignored[0],
+		       (Comparator) strcmp) != NULL;
 }
 
 
@@ -563,7 +575,7 @@ glibtop_get_mountlist_s (glibtop *server, glibtop_mountlist *buf, int all_fs)
 
 	/* Read filesystem list. */
 
-	if((entries = read_filesystem_list (TRUE)) == NULL)
+	if((entries = read_filesystem_list ()) == NULL)
 		return NULL;
 
 	for (cur = &entries[0]; cur != NULL; cur = next) {
@@ -592,5 +604,9 @@ glibtop_get_mountlist_s (glibtop *server, glibtop_mountlist *buf, int all_fs)
 	buf->number = mount_array->len;
 	buf->total  = buf->number * buf->size;
 
+	buf->flags  = (1 << GLIBTOP_MOUNTLIST_SIZE)
+	  | (1 << GLIBTOP_MOUNTLIST_SIZE)
+	  | (1 << GLIBTOP_MOUNTLIST_TOTAL);
+
 	return (glibtop_mountentry*) g_array_free(mount_array, FALSE);
 }
