@@ -1,4 +1,4 @@
-/* $Id: procdata.c,v 1.14 2003/10/19 16:55:46 carlos Exp $ */
+/* $Id: procdata.c,v 1.15 2004/10/11 18:01:25 bdejean Exp $ */
 
 /* Copyright (C) 1998-99 Martin Baulig
    This file is part of LibGTop 1.0.
@@ -53,14 +53,16 @@ glibtop_get_proc_data_psinfo_s (glibtop *server, struct psinfo *psinfo, pid_t pi
 	char buffer [BUFSIZ];
 
 	sprintf (buffer, "/proc/%d/psinfo", (int) pid);
+
 	fd = s_open (buffer, O_RDONLY);
+
 	if (fd < 0) {
 		glibtop_warn_io_r (server, "open (%s)", buffer);
 		return -1;
 	}
 
-	if (s_pread (fd, psinfo, sizeof (struct psinfo), 0) !=
-	             sizeof (struct psinfo))
+	if (s_pread (fd, psinfo, sizeof (struct psinfo), 0)
+	    != sizeof (struct psinfo))
 	{
 		s_close (fd);
 		glibtop_warn_io_r (server, "pread (%s)", buffer);
@@ -78,14 +80,16 @@ glibtop_get_proc_data_usage_s (glibtop *server, struct prusage *prusage, pid_t p
 	char buffer [BUFSIZ];
 
 	sprintf (buffer, "/proc/%d/usage", (int) pid);
+
 	fd = s_open (buffer, O_RDONLY);
+
 	if (fd < 0) {
 		glibtop_warn_io_r (server, "open (%s)", buffer);
 		return -1;
 	}
 
-	if (s_pread (fd, prusage, sizeof (struct prusage), 0) !=
-	             sizeof (struct prusage))
+	if (s_pread (fd, prusage, sizeof (struct prusage), 0)
+	    != sizeof (struct prusage))
 	{
 		s_close (fd);
 		glibtop_warn_io_r (server, "pread (%s)", buffer);
@@ -99,7 +103,7 @@ glibtop_get_proc_data_usage_s (glibtop *server, struct prusage *prusage, pid_t p
 #if LIBGTOP_VERSION_CODE >= 1001002
 int
 glibtop_get_proc_credentials_s(glibtop *server,
-      			       struct prcred *prcred,
+			       struct prcred *prcred,
 			       gid_t *groups,
 			       pid_t pid)
 {
@@ -108,33 +112,35 @@ glibtop_get_proc_credentials_s(glibtop *server,
 	char buffer[BUFSIZ];
 
 	sprintf(buffer, "/proc/%d/cred", (int)pid);
+
 	if((fd = s_open(buffer, O_RDONLY)) < 0)
 	{
-	   	if(errno != EPERM && errno != EACCES)
-		   	glibtop_warn_io_r(server, "open (%s)", buffer);
+		if(errno != EPERM && errno != EACCES)
+			glibtop_warn_io_r(server, "open (%s)", buffer);
 		return -1;
 	}
-	if(s_pread(fd, prcred, sizeof(struct prcred), 0) !=
-	           sizeof(struct prcred))
+
+	if(s_pread(fd, prcred, sizeof(struct prcred), 0)
+	   != sizeof(struct prcred))
 	{
-	   	s_close(fd);
+		s_close(fd);
 		glibtop_warn_io_r(server, "pread (%s)", buffer);
 		return -1;
 	}
+
 	if(prcred->pr_ngroups >= 0)
 	{
-	    if(prcred->pr_ngroups <= GLIBTOP_MAX_GROUPS)
-	        toread = prcred->pr_ngroups * sizeof(gid_t);
-	    else
-	        toread = GLIBTOP_MAX_GROUPS * sizeof(gid_t);
-	    if(s_pread(fd, groups, toread,
-	               &(((struct prcred *)0)->pr_groups[0])) != toread)
-	               prcred->pr_ngroups = 0;
+		toread = MIN(prcred->pr_ngroups, GLIBTOP_MAX_GROUPS) * sizeof(gid_t);
+
+		if((size_t) s_pread(fd,groups, toread,
+				    G_STRUCT_OFFSET(struct prcred, pr_groups))
+		   != toread)
+			prcred->pr_ngroups = 0;
 	}
 	s_close(fd);
 	return 0;
 }
-#endif
+#endif /* LIBGTOP_VERSION_CODE >= 1001002 */
 
 int
 glibtop_get_proc_status_s(glibtop *server, struct pstatus *pstatus, pid_t pid)
@@ -145,14 +151,14 @@ glibtop_get_proc_status_s(glibtop *server, struct pstatus *pstatus, pid_t pid)
 	sprintf(buffer, "/proc/%d/status", (int)pid);
 	if((fd = s_open(buffer, O_RDONLY)) < 0)
 	{
-	   	if(errno != EPERM && errno != EACCES)
-		   	glibtop_warn_io_r(server, "open (%s)", buffer);
+		if(errno != EPERM && errno != EACCES)
+			glibtop_warn_io_r(server, "open (%s)", buffer);
 		return -1;
 	}
-	if(s_pread(fd, pstatus, sizeof(struct pstatus), 0) !=
-	           sizeof(struct pstatus))
+	if(s_pread(fd, pstatus, sizeof(struct pstatus), 0)
+	   != sizeof(struct pstatus))
 	{
-	   	s_close(fd);
+		s_close(fd);
 		glibtop_warn_io_r(server, "pread (%s)", buffer);
 		return -1;
 	}
@@ -164,7 +170,7 @@ glibtop_get_proc_status_s(glibtop *server, struct pstatus *pstatus, pid_t pid)
 
 int
 glibtop_get_proc_data_psinfo_s (glibtop *server,
-      				struct prpsinfo *psinfo,
+				struct prpsinfo *psinfo,
 				pid_t pid)
 {
 	int fd;
@@ -173,7 +179,7 @@ glibtop_get_proc_data_psinfo_s (glibtop *server,
 	sprintf (buffer, "/proc/%d", (int) pid);
 	fd = s_open (buffer, O_RDONLY);
 	if (fd < 0) {
-	   	if(errno != EPERM && errno != EACCES)
+		if(errno != EPERM && errno != EACCES)
 			glibtop_warn_io_r (server, "open (%s)", buffer);
 		return -1;
 	}
@@ -191,7 +197,7 @@ glibtop_get_proc_data_psinfo_s (glibtop *server,
 
 int
 glibtop_get_proc_data_usage_s (glibtop *server,
-      			       struct prusage *prusage,
+			       struct prusage *prusage,
 			       pid_t pid)
 {
 	int fd;
@@ -200,7 +206,7 @@ glibtop_get_proc_data_usage_s (glibtop *server,
 	sprintf (buffer, "/proc/%d", (int) pid);
 	fd = s_open (buffer, O_RDONLY);
 	if (fd < 0) {
-	   	if(errno != EPERM && errno != EACCES)
+		if(errno != EPERM && errno != EACCES)
 			glibtop_warn_io_r (server, "open (%s)", buffer);
 		return -1;
 	}
@@ -218,7 +224,7 @@ glibtop_get_proc_data_usage_s (glibtop *server,
 
 int
 glibtop_get_proc_credentials_s(glibtop *server,
-      			       struct prcred *prcred,
+			       struct prcred *prcred,
 			       gid_t *groups,
 			       pid_t pid)
 {
@@ -229,13 +235,13 @@ glibtop_get_proc_credentials_s(glibtop *server,
 	sprintf(buffer, "/proc/%d", (int)pid);
 	if((fd = s_open(buffer, O_RDONLY)) < 0)
 	{
-	   	if(errno != EPERM && errno != EACCES)
-		   	glibtop_warn_io_r(server, "open (%s)", buffer);
+		if(errno != EPERM && errno != EACCES)
+			glibtop_warn_io_r(server, "open (%s)", buffer);
 		return -1;
 	}
 	if(ioctl(fd, PIOCCRED, prcred) < 0)
 	{
-	   	s_close(fd);
+		s_close(fd);
 		glibtop_warn_io_r(server, "ioctl(%s, PIOCCRED)", buffer);
 		return -1;
 	}
@@ -252,13 +258,13 @@ glibtop_get_proc_status_s(glibtop *server, struct prstatus *pstatus, pid_t pid)
 	sprintf(buffer, "/proc/%d", (int)pid);
 	if((fd = s_open(buffer, O_RDONLY)) < 0)
 	{
-	   	if(errno != EPERM && errno != EACCES)
-		   	glibtop_warn_io_r(server, "open (%s)", buffer);
+		if(errno != EPERM && errno != EACCES)
+			glibtop_warn_io_r(server, "open (%s)", buffer);
 		return -1;
 	}
 	if(ioctl(fd, PIOCSTATUS, pstatus) < 0)
 	{
-	   	s_close(fd);
+		s_close(fd);
 		glibtop_warn_io_r(server, "ioctl(%s, PIOCSTATUS)", buffer);
 		return -1;
 	}
